#! /usr/bin/env python
#
# ESA (C) 2000-2021
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#
#

""" pyset py
 
 This script copies the pysas directory tree in
 $SAS_DIR/lib/python/pysas into the user's development
 environment, below the $SAS_DEV/lib/python/pysas,
 where $SAS_DEV is typically /home/<user>/sasdev.
 
 $SAS_DIR/lib/python/pysas contains the pysas  
 modules and subdirectories, each one for each
 Python sub-package available in SAS. Within
 each sub-package subdirectory, we find all
 the module components of such package.
 
 pysas modules and sub-package's modules
 are all relative softlinks to their real
 counterparts. For pysas these are soft links to
 ../../../packages/pysas/src/pysas.
 For each <pkg> sub-package's sub-directory,
 the soft links go to the respective .py files in 
 ../../../packages/<pkg>/src/<pkg>
 But the sub-package's sub-directories, they are 
 real sub-directories.
 
 Below $SAS_DEV/lib/python/pysas we want
 to replicate the same structure but replacing the
 relative soft links bu absolute soft links to 
 $SAS_DIR/lib/python/pysas.

 This script is run from the SAS Make.include
 under the target 'required_dirs'. 

 To develop packages with SAS, the developer
 must create a directory $SAS_DEV, typically
 $HOME/sasdev, placed ahead of $SAS_DIR, in the
 $SAS_PATH variable, as  $SAS_PATH=$SAS_DEV:$SAS_DIR. 
 Obviously $SAS_DIR is the root directory of 
 the build of reference, required to make any
 package. The $SAS_DIR directory tree includes
 subdirectories bin, lib, include, doc and config, 
 which are filled with the different elements of the
 build: binaries, libraries, headers, documents and
 package configuration files (like the par file).

 The SAS Makefile replicates the $SAS_DIR 
 directory tree in $SAS_DEV by means of the target
 'required_dirs'. It replicates only the directories
 not the files. When the developer builds a package
 the SAS Makefile puts the package binaries 
 in $SAS_DEV/bin), the libraries in $SAS_DEV/lib, etc,
 not as real files but as soft links
 pointing to the real files below the package
 directory tree. Given that $SAS_DEV/bin, $SAS_DEV/lib, 
 etc, are placed ahead of their replicas in PATH,  
 (DY)LD_LIBRARY_PATH, C(PLUS)_INCLUDE_DIR, etc, 
 they will be used instead of those.
 As a consequence, any new binaries, libraries, etc,
 produced by the developer during a local build of
 the package, will be taken and used instead of those 
 already existing below $SAS_DIR, in the build.

 For Python, the mechanism is similar. However, 
 Python packages are all sub-packages of pysas. 
 When a Python package is built, its modules are 
 placed in a subdirectory of 
 $SAS_DEV/lib/python/pysas, named with the 
 package name. This way, it is possible to import it 
 immediately as a subpackge of pysas. 
 Although $SAS_DEV/lib/python/pysas is placed 
 ahead of $SAS_DIR/lib/python/pysas, Python does
 not allow to fall back there to search for a given 
 package if not present in $SAS_DEV/lib/python/pysas.
 Thus, the only solution to allow the user to test
 the whole SAS Python system is to replicate it below
 $SAS_DEV/lib/python.  Such replica is done 
 everytime the make is invoked with target 
 'required_dirs' which also re-creates the rest of 
 the $SAS_DIR/lib/python/pysas directory tree below 
 $SAS_DEV.

 During the testing of Python packages, __pycache__
 subdirectories are created below the 
 $SAS_DEV/lib/python/pysas directory tree. These are
 cleaned during "make clobber"'operations. However, 
 similar directories may appear also below the 
 $SAS_DIR/lib/python/pysas directory tree. These 
 can not be accessed by any user other than sasbuild.
 Therefore, the replication of $SAS_DIR/lib/python/pysas
 into $SAS_DEV/lib/python/pysas must exclude any of these
 subdirectories. 

 __pycache__ subdirectories include compiled Python code
 to ease loading of modules when they are going to run.
 
 The os.walk is used to run scan $SAS_DIR/lib/python/pysas.
 

 SAS_DIR and SAS_PATH must be defined, otherwise
 the procedure exits with an error message.

 SAS_DEV is the first element of SAS_PATH.
 The second is always SAS_DIR. However, we do not
 test here whether both are the same or not.
 """


# Standard library import
import os, sys, shutil


sasdir = os.environ.get('SAS_DIR')
if sasdir == None:
    print('Error: pyset.py: SAS_DIR is undefined')
    sys.exit()

saspath = os.environ.get('SAS_PATH')
if saspath == None:
    print('Error: pyset.py: SAS_PATH is undefined')
    sys.exit()

# sasdev corresponds to SAS_DEV, although this
# variable is not mandatory as are SAS_DIR and SAS_PATH      


sasdev = saspath.split(':')[0]

sasdirlibpysas = os.path.join(sasdir, 'lib/python/pysas')
sasdevlibpysas = os.path.join(sasdev, 'lib/python/pysas')

# Make a fresh new sasdevlibpysas
if os.path.exists(sasdevlibpysas):
    shutil.rmtree(sasdevlibpysas, ignore_errors=True)
    os.mkdir(sasdevlibpysas)

# Just to be able to return where we were
here = os.getcwd()

# Change to the SAS_DIR/lib/python/pysas source directory
os.chdir(sasdirlibpysas)


# os.walk is a generator object which can only be unfolded
# via a for loop.
#
# To scan the directory tree from a given point, we pass it
# as argument ('./'), and select a direction to scan with
# boolean 'topdown'. If True, it will go top-down
# to find directories and files.
#
# Each step of the for loop gives the cur_dir,
# where there could be subdirectories, which are put all into
# the list sub_dirs and files at the same level, also put
# in a list.
#
# For each directory, we remove any __pycache__ subdirectory
# which might be in the sub_dirs list. 
#
# Directories are immediately replicated in sasdevlibpysas.
#
# For each file in the list files, if it is indeed a soft link, 
# we need to find its real full path, absf. This is what we
# will use to create the final soft link in sasdevlibpysas.

for cur_dir, sub_dirs, files in os.walk('./', topdown=True):
    if '__pycache__' in sub_dirs:
        sub_dirs.remove('__pycache__')
    if cur_dir != './':
        newdir = os.path.join(sasdevlibpysas, cur_dir.strip('./'))
        os.mkdir(newdir)

    for f in files:
        fullf = os.path.join(sasdirlibpysas, cur_dir.strip('./'), f)
        if os.path.islink(fullf):
            linkf = os.readlink(fullf)
            absf = os.path.join(os.path.dirname(fullf), linkf)
            absf = os.path.abspath(absf)
            link = os.path.join(sasdevlibpysas, cur_dir.strip('./'), f)
            os.symlink(absf, link)
#
os.chdir(here)
